@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2021 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -39,6 +39,11 @@ require_once($config['base_path'] . '/lib/utility.php');
 
 ini_set('max_execution_time', '0');
 
+/* switch to main database for cli's */
+if ($config['poller_id'] > 1) {
+	db_switch_remote_to_main();
+}
+
 /* process calling arguments */
 $parms = $_SERVER['argv'];
 array_shift($parms);
@@ -52,7 +57,7 @@ $force      = false;
 if (cacti_sizeof($parms)) {
 	foreach($parms as $parameter) {
 		if (strpos($parameter, '=')) {
-			list($arg, $value) = explode('=', $parameter);
+			list($arg, $value) = explode('=', $parameter, 2);
 		} else {
 			$arg = $parameter;
 			$value = '';
@@ -147,6 +152,14 @@ $data_queries = db_fetch_assoc_prepared("SELECT description, hostname, host_id,
 print 'WARNING: Do not interrupt this script.  Reindexing can take quite some time' . PHP_EOL;
 debug("There are '" . cacti_sizeof($data_queries) . "' data queries to run");
 
+/* silently end if the registered process is still running  */
+if (!$force) {
+	if (!register_process_start('reindex', 'master', 0, 86400)) {
+		print "FATAL: Detected an already running process.  Use --force to override" . PHP_EOL;
+		exit(0);
+	}
+}
+
 $i = 1;
 $total_start = microtime(true);
 if (cacti_sizeof($data_queries)) {
@@ -190,6 +203,10 @@ if (cacti_sizeof($data_queries)) {
 
 		$i++;
 	}
+
+	set_config_option('reindex_last_run_time', time());
+	unregister_process('reindex', 'master');
+
 }
 
 function display_version() {
