@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2022 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -98,6 +98,15 @@ if ($auth_method != 0) {
 			if (cacti_sizeof($current_user)) {
 				$_SESSION['sess_user_id'] = $current_user['id'];;
 
+				cacti_log("LOGIN: User '" . $current_user['username'] . "' authenticated via Basic Authentication.", false, 'AUTH');
+
+				$client_addr = get_client_addr();
+
+				db_execute_prepared('INSERT IGNORE INTO user_log
+					(username, user_id, result, ip, time)
+					VALUES (?, ?, 1, ?, NOW())',
+					array($username, $current_user['id'], $client_addr));
+
 				return true;
 			} else {
 				require_once($config['base_path'] . '/auth_login.php');
