@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -482,6 +482,7 @@ function install_setup_get_templates() {
 		'Cacti_Stats.xml.gz',
 		'Cisco_Router.xml.gz',
 		'Citrix_NetScaler_VPX.xml.gz',
+		'Clearpass_policy_manager.xml.gz',
 		'ESXi_Device.xml.gz',
 		'Fortigate.xml.gz',
 		'Generic_SNMP_Device.xml.gz',
@@ -534,8 +535,8 @@ function install_setup_get_templates() {
 			// Loading Template Information from package
 			$myinfo = @json_decode(shell_exec(cacti_escapeshellcmd(read_config_option('path_php_binary')) . ' -q ' . cacti_escapeshellarg($config['base_path'] . '/cli/import_package.php') . ' --filename=' . cacti_escapeshellarg("/$path/$xmlfile") . ' --info-only'), true);
 			$myinfo['filename'] = $xmlfile;
+			$myinfo['name']     = $xmlfile;
 			$info[] = $myinfo;
-			$info[] = array('filename' => $xmlfile, 'name' => $xmlfile);
 		}
 	}
 
@@ -674,43 +675,43 @@ function install_file_paths() {
 	/* RRDtool Binary Path */
 	$input['path_rrdtool'] = install_tool_path('rrdtool',
 		array(
-			'unix'  => '/usr/local/bin/rrdtool',
+			'unix'  => '/usr/bin/rrdtool',
 			'win32' => 'c:/rrdtool/rrdtool.exe'
 		));
 
 	/* snmpwalk Binary Path */
 	$input['path_snmpwalk'] = install_tool_path('snmpwalk',
 		array(
-			'unix'  => '/usr/local/bin/snmpwalk',
-			'win32' => 'c:/net-snmp/bin/snmpwalk.exe'
+			'unix'  => '/usr/bin/snmpwalk',
+			'win32' => 'c:/usr/bin/snmpwalk.exe'
 		));
 
 	/* snmpget Binary Path */
 	$input['path_snmpget'] = install_tool_path('snmpget',
 		array(
-			'unix'  => '/usr/local/bin/snmpget',
-			'win32' => 'c:/net-snmp/bin/snmpget.exe'
+			'unix'  => '/usr/bin/snmpget',
+			'win32' => 'c:/usr/bin/snmpget.exe'
 		));
 
 	/* snmpbulkwalk Binary Path */
 	$input['path_snmpbulkwalk'] = install_tool_path('snmpbulkwalk',
 		array(
-			'unix'  => '/usr/local/bin/snmpbulkwalk',
-			'win32' => 'c:/net-snmp/bin/snmpbulkwalk.exe'
+			'unix'  => '/usr/bin/snmpbulkwalk',
+			'win32' => 'c:/usr/bin/snmpbulkwalk.exe'
 		));
 
 	/* snmpgetnext Binary Path */
 	$input['path_snmpgetnext'] = install_tool_path('snmpgetnext',
 		array(
-			'unix'  => '/usr/local/bin/snmpgetnext',
-			'win32' => 'c:/net-snmp/bin/snmpgetnext.exe'
+			'unix'  => '/usr/bin/snmpgetnext',
+			'win32' => 'c:/usr/bin/snmpgetnext.exe'
 		));
 
 	/* snmptrap Binary Path */
 	$input['path_snmptrap'] = install_tool_path('snmptrap',
 		array(
-			'unix'  => '/usr/local/bin/snmptrap',
-			'win32' => 'c:/net-snmp/bin/snmptrap.exe'
+			'unix'  => '/usr/bin/snmptrap',
+			'win32' => 'c:/usr/bin/snmptrap.exe'
 		));
 
 	/* sendmail Binary Path */
@@ -875,6 +876,24 @@ function remote_update_config_file() {
 	return $failure;
 }
 
+/**
+ * set_install_config_option - Set a config option into the local database only
+ *
+ * @param $config_name - the name of the configuration setting as specified $settings array
+ * @param $value       - the values to be saved
+ *
+ * @return null        - nothing is returned
+ */
+function set_install_config_option($name, $value) {
+	global $local_db_cnn_id;
+
+	if (is_object($local_db_cnn_id)) {
+		db_execute_prepared('REPLACE INTO settings (name, value) VALUES (?, ?)', array($name, $value), false, $local_db_cnn_id);
+	} else {
+		set_config_option($name, $value);
+	}
+}
+
 function import_colors() {
 	global $config;
 
