@@ -2,7 +2,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2023 The Cacti Group                                 |
+ | Copyright (C) 2004-2024 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -901,6 +901,30 @@ while ($poller_runs_completed < $poller_runs) {
 	}
 }
 
+function poller_heartbeat_check() {
+	$poller_interval = read_config_option('poller_interval');
+
+	$heartbeat_pollers = db_fetch_assoc_prepared('SELECT *, UNIX_TIMESTAMP() - UNIX_TIMESTAMP(last_status) AS heartbeat
+		FROM poller
+		WHERE disabled = ""
+		HAVING heartbeat > ? * 2
+		OR status = 6',
+		array($poller_interval));
+
+	if (cacti_sizeof($heartbeat_pollers)) {
+		foreach($heartbeat_pollers as $p) {
+			db_execute_prepared('UPDATE poller SET status = 6 WHERE id = ?', array($p['id']));
+
+			if (debounce_run_notification('poller_heartbeat_' . $p['id'], 1800)) {
+				$log_message = sprintf('WARNING: Poller[%s] with Name:%s is in Heartbeat Status', $p['id'], $p['name']);
+				$email_message = __('WARNING: PollerID:%s with Name:%s is in Heartbeat Status', $p['id'], $p['name']);
+				cacti_log($log_message, false, 'POLLER');
+				admin_email(__('Poller in Heartbeat Mode'), $email_message);
+			}
+		}
+	}
+}
+
 /* start post data processing */
 if ($poller_id == 1) {
 	multiple_poller_boost_check();
@@ -917,6 +941,7 @@ if ($poller_id == 1) {
 	api_plugin_hook('poller_bottom');
 	bad_index_check($mibs);
 	host_status_cache_check();
+	poller_heartbeat_check();
 } else {
 	// flush the boost table if in recovery mode
 	if ($poller_id > 1 && $config['connection'] == 'recovery') {
@@ -1127,13 +1152,14 @@ function log_cacti_stats($loop_start, $method, $concurrent_processes, $max_threa
 			(id, total_time, min_time, max_time, avg_time, total_polls, last_update, last_status, status)
 			VALUES (?, ?, ?, ?, ?, ?, NOW(), NOW(), 2)
 			ON DUPLICATE KEY UPDATE
-			total_time=VALUES(total_time),
-			min_time=VALUES(min_time),
-			max_time=VALUES(max_time),
-			avg_time=VALUES(avg_time),
-			total_polls=VALUES(total_polls),
-			last_update=VALUES(last_update),
-			last_status=VALUES(last_status), status=VALUES(status)',
+			total_time = VALUES(total_time),
+			min_time = VALUES(min_time),
+			max_time = VALUES(max_time),
+			avg_time = VALUES(avg_time),
+			total_polls = VALUES(total_polls),
+			last_update = VALUES(last_update),
+			last_status = VALUES(last_status),
+			status = VALUES(status)',
 			array($poller_id, round($total_time, 4), $min_time, $max_time, $avg_time, $total_polls + 1), true, $poller_db_cnn_id);
 	}
 
